package cmd

import (
	"io"
	"os"

	"github.com/rs/zerolog"
	"golang.org/x/crypto/ssh/terminal"

	"orus.io/orus-io/rednerd/utils"
)

var (
	logOutput   io.Writer = os.Stdout
	logWrappers []func(io.Writer) io.Writer

	log = zerolog.
		New(os.Stdout).
		With().
		Timestamp().
		Logger().
		Level(zerolog.WarnLevel)
)

func resetOutput() {
	out := logOutput
	for _, wrapper := range logWrappers {
		out = wrapper(out)
	}
	log = log.Output(out)
}

func setMinLoggingLevel(level zerolog.Level) {
	if level < log.GetLevel() {
		log = log.Level(level)
	}
}

func (o *Options) setupLogging() {
	var logLevelAutoLocked = false

	o.LogFormat = func(format string) {
		if format == "auto" {
			if terminal.IsTerminal(int(os.Stdout.Fd())) {
				format = "pretty"
			} else {
				format = "json"
			}
		}
		switch format {
		case "pretty":
			logOutput = utils.ConsoleWriter{Out: os.Stdout}
		case "json":
			logOutput = os.Stdout
		default:
			log.Fatal().Msg("Invalid log-format")
		}
		resetOutput()
	}
	o.Verbose = func() {
		log = log.Level(log.GetLevel() - zerolog.Level(1))
	}
	o.LogLevel = func(value string) error {
		if value == "auto" {
			if logLevelAutoLocked {
				// The current call is at best redondant, at worse called by
				// default after some potential --verbose that would be ignored
				return nil
			}
			if terminal.IsTerminal(int(os.Stdout.Fd())) {
				value = "info"
			} else {
				value = "warn"
			}
		}

		level, err := zerolog.ParseLevel(value)
		if err != nil {
			return err
		}
		log = log.Level(level)
		return nil
	}
	o.LogFormat("auto")
	err := o.LogLevel("auto")
	if err != nil {
		panic(err)
	}
	logLevelAutoLocked = true
}
