package wpd

import (
	"io"
	"os/exec"
)

// RealRunner runs an actual wpd
type RealRunner struct {
	cmd    *exec.Cmd
	stdin  io.WriteCloser
	stdout io.ReadCloser
}

// Stdin returns the process input
func (rr *RealRunner) Stdin() io.WriteCloser {
	return rr.stdin
}

// Stop stops the process
func (rr *RealRunner) Stop() error {
	rr.stdin.Close()
	return rr.cmd.Wait()
}

// Stdout returns the process output
func (rr *RealRunner) Stdout() io.ReadCloser {
	return rr.stdout
}

// NewRealRunner ...
func NewRealRunner() (*RealRunner, error) {
	cmd := exec.Command("wpd")

	in, err := cmd.StdinPipe()
	if err != nil {
		return nil, err
	}

	out, err := cmd.StdoutPipe()
	if err != nil {
		return nil, err
	}

	if err := cmd.Start(); err != nil {
		return nil, err
	}
	return &RealRunner{
		cmd:    cmd,
		stdin:  in,
		stdout: out,
	}, nil
}

// RealRunnerFactory ...
func RealRunnerFactory() (Runner, error) {
	runner, err := NewRealRunner()
	return runner, err
}
